import { License } from '../integration/index.js';
import { Request, UrlTools } from '../util/index.js';
import { DockerAssistant } from './index.js';
import ms from 'ms';
class Login {
    constructor() {
        this.defaultProcotol = 'http';
        this.maxRetryAttempts = 3;
        this.intervalAttempts = ms('5s');
    }
    isValidUser(user) {
        if (!user || !user.ID)
            throw new Error('INVALID_USER');
    }
    async requestToken(route, params, retryCount = 0) {
        return Request.post(route, params).then(({ token }) => token).catch((err) => {
            if (retryCount < this.maxRetryAttempts) {
                return new Promise((resolve, reject) => {
                    setTimeout(() => {
                        this.requestToken(route, params, retryCount + 1)
                            .then((response) => resolve(response))
                            .catch((error) => reject(error));
                    }, this.intervalAttempts);
                });
            }
            return Promise.reject(err);
        });
    }
    async getToken(serverHost, { CDOPERADOR, NMOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/generateToken`;
        const token = await this.requestToken(route, { CDOPERADOR, NMOPERADOR });
        return token;
    }
    async getAccessProfile(serverHost, { CDOPERADOR, NRORG }) {
        const dns = UrlTools.getUrlWithProtocol(serverHost);
        const route = `${dns}:${NRORG}/API/accessProfile`;
        const accessProfile = await Request.post(route, { CDOPERADOR });
        return accessProfile;
    }
    async setServerForOrganization(params) {
        return License.setServerForOrganization(params);
    }
    async login({ email, password, agent }) {
        let errorInfo = {};
        const defaultErrorHandler = (e, errorCode) => {
            errorInfo = {
                message: e.message,
                stack: e.stack,
            };
            throw new Error(errorCode);
        };
        try {
            const response = await License.auth({ email, password }).catch((e) => {
                defaultErrorHandler(e, 'ERR_CALL_LICENSE');
            });
            if (!response.dataset) {
                errorInfo = {
                    message: response.error || response.message || 'Unexpected Error',
                    stack: 'call license',
                    data: response,
                };
                throw new Error('ERR_CALL_LICENSE');
            }
            const { dataset: { operador } } = response;
            const { NRORG, ID: CDOPERADOR, NAME: NMOPERADOR } = operador;
            this.isValidUser(operador);
            const { dataset: { serverUrl: server } } = await License.getServerByOrganization({ NRORG });
            const { SERVER_DNS = '' } = server;
            let token = '';
            const ACCESS_PROFILE = { PERMISSIONS: '', error: false, errorMessage: '' };
            const serverInfo = await DockerAssistant.handleClientServer(NRORG, SERVER_DNS).catch((e) => {
                defaultErrorHandler(e, 'ERR_CREATE_CONTAINER_ORG');
            });
            if (!SERVER_DNS && serverInfo.serverName) {
                await this.setServerForOrganization({
                    NRORG,
                    HOST: serverInfo.host,
                });
            }
            if (agent !== 'assistant-installer') {
                token = await this.getToken(SERVER_DNS || serverInfo.host, { CDOPERADOR, NMOPERADOR, NRORG }).catch((e) => {
                    defaultErrorHandler(e, 'ERR_GENERATE_TOKEN');
                });
                await this.getAccessProfile(SERVER_DNS || '', { CDOPERADOR }).then((accessProfile) => {
                    Object.assign(ACCESS_PROFILE, accessProfile);
                }).catch((e) => {
                    ACCESS_PROFILE.error = true;
                    ACCESS_PROFILE.errorMessage = e.message;
                });
            }
            return {
                STATUS: true,
                STATUS_MSG: 'SUCCESS',
                STATUS_DATA: {},
                NODE: SERVER_DNS,
                NRORG,
                TOKEN: token,
                CDOPERADOR,
                NMOPERADOR,
                ACCESS_PROFILE,
            };
        }
        catch (error) {
            return {
                STATUS: false,
                STATUS_MSG: error.message,
                STATUS_DATA: errorInfo,
            };
        }
    }
}
export default new Login();
