import path from 'path';
import fs from 'fs';
import dotenv from 'dotenv';
import { createHash } from 'crypto';

const backendPath = path.dirname(path.join(process.argv[1], '..'));

class BuildEnv {
  static DEFAULT_VALUES = {
    UPDATER_VERSIONS_PATH: path.join(backendPath, 'temp'),
  }

  static async fileExists(filePath) {
    return new Promise((resolve) => {
      fs.access(filePath, fs.constants.F_OK, (err) => {
        if(err) return resolve(false);
        resolve(true);
      });
    })
  }

  static async readFile(filePath) {
    return new Promise((resolve, reject) => {
      fs.readFile(filePath, 'utf8', (err, data) => {
        if(err) return reject(new Error(`Erro ao ler o arquivo: ${filePath}`));
        resolve(data);
      });
    })
  }

  static async writeFile(filePath, content) {
    return new Promise((resolve, reject) => {
      fs.writeFile(filePath, content, (err) => {
        if(err) return reject(`Erro ao escrever no arquivo: ${filePath}`);
        return resolve();
      })
    });
  }
  
  static isDevMode() {
    const { NODE_ENV = 'development' } = process.env;
    const executionMode = ['dev', 'prod'].includes(process.argv[2]) ? process.argv[2] : null;
    return executionMode === 'dev' || (!executionMode && NODE_ENV === 'development');
  }

  static replaceMnemonics(text, params = {}) {
    return Object.keys(params).reduce((textToReplace, paramName) => {
      textToReplace = textToReplace.replace(`<>${paramName}<>`, params[paramName]);
      return textToReplace;
    }, text);
  }

  static async build() {
    try {
      const forceBuild = process.argv[3] === 'force';
      const envFile = path.join(backendPath, '.env');
      const envProdFile = path.join(backendPath, '.env.production');
      const envDevFile = path.join(backendPath, '.env.development');
      const envExampleFile = path.join(backendPath, '.env.example');
      if (!forceBuild) {
        const fileExists = await BuildEnv.fileExists(envFile);
        if(fileExists) return;
      }
      let envExampleStr = await BuildEnv.readFile(envExampleFile);
      envExampleStr = envExampleStr.replace(/#{2,}/gm, 'HEADER_COMMENT');
      envExampleStr = envExampleStr.replace(/#.*/gm, ''); // remove line comments
      envExampleStr = envExampleStr.replace(/HEADER_COMMENT/g, '#');
      let envStr = envExampleStr;
      let fileNameDefaultValues = BuildEnv.isDevMode() ? envDevFile : envProdFile;
      const envValuesStr = await BuildEnv.readFile(fileNameDefaultValues);
      let defaultValues = dotenv.parse(envValuesStr);
      const privateKey = createHash('sha256');
      const privateKeyHex = privateKey.digest('hex');
      if(BuildEnv.isDevMode()) {
        defaultValues = { ...defaultValues, ...BuildEnv.DEFAULT_VALUES };
      }
      defaultValues.SECRET_KEY = privateKeyHex;
      envStr = BuildEnv.replaceMnemonics(envExampleStr, defaultValues);
      BuildEnv.writeFile(envFile, envStr);
    } catch(error) {
      console.error(error.message);
    }
  }
}



BuildEnv.build();
